from typing import List, Tuple
from django import forms

from django.apps import apps


class DBSchemaSelectionForm(forms.Form):
    # * Get apps with models for selection
    stateful_apps = [app.label for app in apps.get_app_configs() if app.models]
    # * Get model list
    model_list = [app for app in apps.get_models()]
    app_choices: List[Tuple[str, str]] = sorted(
        set(
            (app_name, app_name)
            for app_name in stateful_apps
            if next(apps.get_app_config(app_name).get_models(), None) is not None
        )
    )
    model_choices: List[Tuple[str, str]] = sorted(
        set((model_name._meta.object_name, model_name._meta.object_name) for model_name in model_list)
    )
    selected_apps = forms.MultipleChoiceField(
        choices=app_choices,
        required=False,
        label="Selected apps",
        widget=forms.CheckboxSelectMultiple,
    )
    selected_models = forms.MultipleChoiceField(
        required=False,
        choices=model_choices,
        label="Selected models",
        widget=forms.CheckboxSelectMultiple,
    )



from tempfile import TemporaryDirectory
from django.http import HttpResponse
from django.shortcuts import render
from django.core.management import call_command
from django_db_schema_renderer.forms import DBSchemaSelectionForm
from urllib.parse import quote

# TODO: Find some pretty svg
SVG = "%3Csvg xmlns='http://www.w3.org/2000/svg' %3E%3Crect width='400' height='300' \
    fill='%23cccccc'%3E%3C/rect%3E%3Ctext %3E400x300%3C/text%3E%3C/svg%3E"


def create_db_schema_graph_model_view(request) -> HttpResponse:
    """
    Renders a checkbox form or an SVG.

    Once the form is submitted, this function creates a temporary output file,
        calls the django extension "graph_models" command, and then sends the
        SVG to the output template.

    Args:
        request (WSGIRequest)

    Returns:
        An HttpResponse object which renders the selection or visualization page.
    """
    if request.method == "POST":
        form = DBSchemaSelectionForm(request.POST)
        if form.is_valid():
            apps = form.cleaned_data["selected_apps"]
            models = form.cleaned_data["selected_models"]
            with TemporaryDirectory() as tmpdir:
                _output_filepath: str = f"{tmpdir}/db_schema.svg"
                # * Empty form render all db schema
                if not apps and not models:
                    call_command(
                        "graph_models",
                        all_applications=True,
                        group_models=True,
                        output=_output_filepath,
                        arrow_shape="Normal",
                    )
                # * Only apps provided, render all models for apps
                if apps and not models:
                    call_command(
                        "graph_models",
                        apps,
                        group_models=True,
                        output=_output_filepath,
                        arrow_shape="Normal",
                    )
                # * Only models provided , apps became we must use  all apps to hit models
                if models:
                    call_command(
                        "graph_models",
                        all_applications=True,
                        include_models=models,
                        group_models=True,
                        output=_output_filepath,
                        arrow_shape="Normal",
                    )
                with open(_output_filepath, "r") as file_svg:
                    svg: str = file_svg.read()
                    svg: str = svg[svg.find("<svg") :]
                    svg: str = svg.replace(' xlink:title="&lt;TABLE&gt;"', "")
                    svg: str = quote(svg)
            return render(request=request, template_name="input.html", context={"form": form, "svg": svg})
    else:
        form = DBSchemaSelectionForm()
    return render(
        request=request,
        template_name="input.html",
        context={
            "form": form,
            "svg": SVG,
        },
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_db_schema_renderer']

package_data = \
{'': ['*'], 'django_db_schema_renderer': ['templates/*', 'templates/admin/*']}

install_requires = \
['Django>=3.2', 'django-extensions>=3.2.1,<4.0.0', 'pygraphviz>=1.10,<2.0']

setup_kwargs = {
    'name': 'django-db-schema-renderer',
    'version': '0.1.0',
    'description': 'Django app to render ER diagram',
    'long_description': '=============================\ndjango-db-schema-renderer app\n=============================\n\nApp to draw db-schema of selected apps, models.\n\nDetailed documentation is in the "docs" directory.\n\nQuick start\n-----------\n\n1. Add "django-db-schema-renderer" to your INSTALLED_APPS setting like this::\n    INSTALLED_APPS = [\n        \'django_db-schema_renderer\',\n        \'django.contrib.admin\',\n\n    ]\n\n\n2. Add app related urls to your root `urls.py`\n    from django_db_schema_renderer.urls import schema_urls\n    \n    urlpatterns = [\n    ...\n    path("db-schema/", include((schema_urls, "db-schema"))),\n    ...\n\n    ]\n\n3. Start the development server and visit http://127.0.0.1:8000/admin/\n   to render db-schema.\n\n',
    'author': 'Oleksandr Korol',
    'author_email': 'oleksandr.korol@coaxsoft.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
